import { __awaiter, __generator } from "tslib";
import { UAParser } from 'ua-parser-js';
export { default as withMockProperties } from './mock_properties';
export { default as withMockMatchMedia } from './mock_match_media';
/*
 * Sometimes tests need to know what browser they run in to make proper assertions.
 * Karma doesn't provide this information.
 * The browser detect functions in the `src` directory can't be used because they are objects of testing.
 * Therefore a popular third party library is used to detect browser.
 * The library isn't used in the main code because some visitors tamper user agent while the test environments don't.
 *
 * We should find a way to pass browser settings from the Karma configuration to the tests,
 * otherwise we can't distinguish incognito browsers from regular browsers, Brave from Chrome, etc.
 */
export function isTrident() {
    return new UAParser().getEngine().name === 'Trident';
}
export function isEdgeHTML() {
    return new UAParser().getEngine().name === 'EdgeHTML';
}
export function isChromium() {
    return new UAParser().getEngine().name === 'Blink';
}
export function isGecko() {
    return new UAParser().getEngine().name === 'Gecko';
}
export function isWebKit() {
    return new UAParser().getEngine().name === 'WebKit';
}
export function isMobile() {
    return new UAParser().getDevice().type === 'mobile';
}
export function isTablet() {
    return new UAParser().getDevice().type === 'tablet';
}
export function isAndroid() {
    return new UAParser().getOS().name === 'Android';
}
export function isWindows() {
    return new UAParser().getOS().name === 'Windows';
}
export function isMacOS() {
    return new UAParser().getOS().name === 'Mac OS';
}
/**
 * Probably you should use `isWebKit` instead
 */
export function isSafari() {
    var browserName = new UAParser().getBrowser().name;
    return browserName === 'Safari' || browserName === 'Mobile Safari';
}
/**
 * Probably you should use `getBrowserEngineMajorVersion` instead
 */
export function getBrowserMajorVersion() {
    var version = new UAParser().getBrowser().version;
    if (version === undefined) {
        return undefined;
    }
    return parseInt(version.split('.')[0]);
}
export function getBrowserEngineMajorVersion() {
    var version = new UAParser().getEngine().version;
    if (version === undefined) {
        return undefined;
    }
    return parseInt(version.split('.')[0]);
}
export function withCSS(css, action) {
    var _a;
    return __awaiter(this, void 0, void 0, function () {
        var styleElement;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    styleElement = document.createElement('style');
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, , 3, 4]);
                    styleElement.textContent = css;
                    document.head.appendChild(styleElement);
                    return [4 /*yield*/, action()];
                case 2: return [2 /*return*/, _b.sent()];
                case 3:
                    (_a = styleElement.parentNode) === null || _a === void 0 ? void 0 : _a.removeChild(styleElement);
                    return [7 /*endfinally*/];
                case 4: return [2 /*return*/];
            }
        });
    });
}
export function holdLoop(timeMs) {
    var startTime = Date.now();
    while (Date.now() < startTime + timeMs) {
        // Do nothing
    }
}
